package com.sams.jws.chapter15;
/*
 * HelloWorldReceiver
 * Receive a JAXM message and reply in the time-honored fashion.
 */

// General JDK
import java.io.IOException;
// JAXM
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPPart;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPException;
import javax.xml.messaging.JAXMServlet;
import javax.xml.messaging.ReqRespListener;
// Servlet
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;

public class HelloWorldReceiver
    extends JAXMServlet
    implements ReqRespListener {
    MessageFactory messageFactory = null;
    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);
        try {
            messageFactory = MessageFactory.newInstance();
        } catch (SOAPException e) {
            System.err.println("Exception caught instantiating MessageFactory:");
            e.printStackTrace();
        } // Catch
    }
    public SOAPMessage onMessage(SOAPMessage requestSoapMsg) {
        System.out.println(
            "HelloWorldReceiver recv the following JAXM msg in onMessage():");
        try {
            requestSoapMsg.writeTo(System.out);
            System.out.println("\n");

            SOAPMessage responseSoapMsg = messageFactory.createMessage();

            /*
             * Note that a header element is not defined here. 
             * Although this is an optional element, JAXM will implicitly 
             * nevertheless create an empty header.
             */
            // Create Body element containing response.
            SOAPEnvelope responseSoapEnv = responseSoapMsg.getSOAPPart()
                                                          .getEnvelope();
            SOAPBody responseSoapBody = responseSoapEnv.getBody();
            SOAPBodyElement responseSoapBodyEl =
                responseSoapBody.addBodyElement(responseSoapEnv.createName("Response"));
            responseSoapBodyEl.addTextNode("Hello world");

            responseSoapMsg.saveChanges();
            System.out.println(
                "HelloWorldReceiver responding with the following JAXM message:");
            responseSoapMsg.writeTo(System.out);
            System.out.println("\n");

            return responseSoapMsg;

        } catch (SOAPException e) {
            System.err.println("Error in processing or replying to a message");
            e.printStackTrace();
            return null;
        } catch (IOException e) {
            System.err.println(
                "Encountered IOException writing SOAPMessage to System.out: ");
            e.printStackTrace();
            return null;
        }
    } // onMessage()
} // HelloWorldReceiver