import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.security.interfaces.*;

/** Uses the public key from a certificate (this is, it uses the
 *  original data file and the two files generated by the SignData
 *  program) to verify a digital signature against a file.
 */

public class VerifyData
{
    public static void main(String[] args)
    {
        try
        {
            if (args.length < 3)
            {
                System.out.println(
                    "Please supply the data file, the sig file "+
                    "and the cert file.");
                System.exit(1);
            }

            FileInputStream certFile = new FileInputStream(args[2]);

// Create a certificate factory to read an X.509 certificate.
            CertificateFactory certFact = CertificateFactory.
                getInstance("X.509");

// Load the certificate from the certificate file.
            java.security.cert.Certificate cert =
                certFact.generateCertificate(certFile);

            certFile.close();

// Extract the public key from the certificate (assume it's an RSA key).
            RSAPublicKey pubKey = (RSAPublicKey) cert.getPublicKey();

// Create and initialize a Signature object for MD5 and RSA.
            Signature sigVerifier = Signature.getInstance("MD5withRSA");
            sigVerifier.initVerify(pubKey);

// Open the data file.
            FileInputStream dataFile = new FileInputStream(args[0]);

// Create a buffer for reading the file.
            byte[] buffer = new byte[4096];

            int len;

// Read block of bytes from the file and add them to the signature.
            while ((len = dataFile.read(buffer)) > 0)
            {
                sigVerifier.update(buffer, 0, len);
            }

            dataFile.close();

// Create a byte buffer for the original signature.
            File sigFile = new File(args[1]);
            byte sigBytes[] = new byte[(int) sigFile.length()];

// Read the signature from a file.
            FileInputStream sigIn = new FileInputStream(sigFile);
            sigIn.read(sigBytes);
            sigIn.close();

// Compare the signature from the file with the data file
// and the public key from the certificate.
            if (sigVerifier.verify(sigBytes))
            {
                System.out.println("The signature matches.");
            }
            else
            {
                System.out.println(
                    "The signature doesn't match.");
            }
        }
        catch (Exception exc)
        {
            exc.printStackTrace();
        }
    }
}
